module Video (
	Reset,
	PixelsValid,
	Pixel0In,
	Pixel1In,
	Address,
	AddressValid,
	FrameBufferClkIn,
	VideoClkIn,
	
	VideoOut,
	VideoReady,
	VideoValid
);

	input Reset;
	input FrameBufferClkIn, VideoClkIn;
	
	input [15:0] 	Pixel0In,Pixel1In;
	input 		 	PixelsValid;
	output [17:0]	Address;
	output 			AddressValid;

	output [23:0] VideoOut;
	input 		  VideoReady;
	output		  VideoValid;
	
	wire PixelFIFOReadyForWrite;
	wire fifo_almost_full;
	wire fifo_full;
	wire fifo_valid;
	wire [15:0] PixelUnpadded;
	
	assign PixelFIFOReadyForWrite = ~(fifo_almost_full|fifo_full);
	
	//--------------------------------------------------------
	// AddressGenerator
	//--------------------------------------------------------
	SequentialAddressGenerator addrGen (
		.Clock(FrameBufferClkIn), 
		.Reset(Reset), 
		.Next(PixelFIFOReadyForWrite), 
		.AddressOut(Address), 
		.AddressValid(AddressValid));
	//--------------------------------------------------------
	
	
	
	//assign VideoValid = 1'b1; // NOTE: constant 1 okay
	assign VideoValid = fifo_valid;
	
FBReadFifo pixfifo (
	.din({Pixel0In,Pixel1In}), // Bus [31 : 0] // switch so that read out even pixel first
	.rd_clk(VideoClkIn),
	.rd_en(VideoReady),
	.rst(Reset),
	.wr_clk(FrameBufferClkIn),
	.wr_en(PixelsValid),
	.almost_full(fifo_almost_full),
	.dout(PixelUnpadded), // Bus [15 : 0] 
    .valid(fifo_valid),
	.full(fifo_full));
	
	
	// RGB padding R5, G6, B5
	PixelAdapter pixAdapter (.InPixel(PixelUnpadded), .OutPixel(VideoOut));

endmodule
